'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createContainer;

var _utils = require('./utils');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * @typedef   {Object}    Container
 * @property  {Function}  add
 * @property  {Function}  share
 * @property  {Function}  get
 */

/**
 * @return {Container}
 */
function createContainer() {
  // eslint-disable-line max-statements
  var dependencies = [];

  /**
   * Registers a depedency at the container.
   *
   * @param {string}    name      Dependency name
   * @param {Function}  func      Constructor or factory function
   * @param {Array}     [args=[]] Dependency arguments to inject
   */
  function add(name, func) {
    var args = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    validateRegistrationArguments(name, func, args, 'add');
    var dependency = { name: name, func: func, args: args };
    dependencies = [].concat(_toConsumableArray(dependencies), [dependency]);
  }

  /**
   * Registers a singleton depedency at the container.
   *
   * @param {string}    name      Dependency name
   * @param {Function}  func      Constructor or factory function
   * @param {Array}     [args=[]] Dependency arguments to inject
   */
  function share(name, func) {
    var args = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    validateRegistrationArguments(name, func, args, 'share');
    var dependency = { name: name, func: func, args: args, shared: true };
    dependencies = [].concat(_toConsumableArray(dependencies), [dependency]);
  }

  /**
   * Get an instantiated, injected dependency from the container.
   *
   * @param {string}  name  Dependency name
   * @return {*}  Resolved dependency
   */
  function get(name) {
    (0, _utils.validateArguments)([name], 'get', { name: ['string', true] });
    var dependency = findDependency(name);
    (0, _utils.invariant)(dependency, 'Dependency \'' + name + '\' does not exist.');
    return instantiate(dependency);
  }

  function instantiate(_ref) {
    var name = _ref.name,
        func = _ref.func,
        instance = _ref.instance,
        args = _ref.args,
        shared = _ref.shared;

    if (shared) {
      return instance || createSingle(name, func, instance, args);
    }

    return create(func, args);
  }

  function create(func, args) {
    var resolvedArgs = args.map(get);
    return func.hasOwnProperty('prototype') ? new (Function.prototype.bind.apply(func, [null].concat(_toConsumableArray(resolvedArgs))))() : // eslint-disable-line new-cap
    func.apply(undefined, _toConsumableArray(resolvedArgs));
  }

  function createSingle(name, func, instance, args) {
    var newInstance = create(func, args);
    dependencies = setInstance(name, newInstance);
    return newInstance;
  }

  function findDependency(name) {
    return dependencies.find(function (dep) {
      return dep.name === name;
    });
  }

  function setInstance(name, instance) {
    return dependencies.map(function (dep) {
      return name === dep.name ? _extends({}, dep, { instance: instance }) : dep;
    });
  }

  function validateRegistrationArguments(name, func, args, type) {
    (0, _utils.validateArguments)([name, func, args], type, {
      name: ['string', true],
      constructor: ['function', true],
      arguments: ['array']
    });
    (0, _utils.invariant)(!findDependency(name), 'Dependency \'' + name + '\' already exists.');
  }

  return { add: add, share: share, get: get };
}
module.exports = exports['default'];